﻿/**
*@FILE  		: MainWindow.xaml.cs
*@DESCRIPTION	: Program that emulates the stock market
*@              : by accepting investment wireless from
*@              : multiple mobile devices.
*@              : DECEMBER 16, 2015
*@VERSION		: 1.0.0
*@PROGRAMMER	: BRIAN HINDS & JAMES SIMPSON
*/

using DevExpress.Xpf.Charts;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Threading;
using System.Windows;
using System.Windows.Media;

namespace PROG2120_Project
{
    /**
    *@CLASS			    :MainWindow : Window
    *@DESCRIPTION		:Interaction logic for MainWindow.xaml
    */

    public partial class MainWindow : Window
    {
        /**
         *@ Variable for transactions.
         */
        private int companyA;
        private int companyB;
        private int companyC;
        private int companyD;
        private int companyE;
        /**
        *@ Variable for investment amount.
        */
        private long valueA;
        private long valueB;
        private long valueC;
        private long valueD;
        private long valueE;
        /**
        *@ Variable for total investment.
        */
        private long valueA1;
        private long valueB1;
        private long valueC1;
        private long valueD1;
        private long valueE1;
        /**
        *@ Variable for investors' transactions.
        */
        private string _CompID;
        private string _CompName;
        private string _Investor;
        private string _TransactionAmount;
        /**
        *@ Variable for total company's profits and net value.
        */
        public double aValue;
        public double aProfit;
        public double bValue;
        public double bProfit;
        public double cValue;
        public double cProfit;
        public double dValue;
        public double dProfit;
        public double eValue;
        public double eProfit;
        /**
        *@ Variable for user who invest.
        */
        private string user;
        /**
         *@ Hash-table for keeping track of investors and their calculated profits
         *@ before entering into database. This also prevents duplicates.
         */
        private Hashtable records = new Hashtable();
        private Hashtable banking = new Hashtable();
        /**
        *@ Objects for dataset
        */
        private StockMarketDataSet.TransactionInfoDataTable transactionInfoTable = new StockMarketDataSet.TransactionInfoDataTable();
        /**
        *@METHOD		:MainWindow()
        *@DESCRIPTION	:Initialized the components for the application
        */

        public MainWindow()
        {
            InitializeComponent();
            /**
            *@ Start server connection.
            */
            Start.StartServerConnection();
            /**
             *@ Display connection information.
             */
            txtinfo.Text = Start._Information;
            /**
            *@ Window background color.
            */
            Background = new SolidColorBrush(Colors.Green);
            /**
            *@ Loads graph with data.
            */
            TheChart();

            /**
            *@ Set up the dataset
            */
        }

        /**
        *@METHOD		:Window_Loaded()
        *@DESCRIPTION	:Load the components for the application
        */

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            ReadTransactionInfo();
            txtLock();
        }

        /**
        *@METHOD		:Window_Closing()
        *@DESCRIPTION	:Execute methods up on form close.
        */

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            WriteTransactionInfo();
        }

        private void Start_Loaded(object sender, RoutedEventArgs e)
        {
        }

        /**
        *@METHOD			:serverControl
        *@DESCRIPTION		:This method handles the condition when the client
        *@                  :make a letter selection.
        *@PARAMETERS		:object sender, RoutedEventArgs e1)
        *@RETURNS			:None
        */

        private void serverControl(object sender, RoutedEventArgs e1)
        {
            /**
            *@ Argument from the server class.
            */
            var _Event = e1 as Server.RemoteControlEventArgs;
            /**
             *@ Code for the company to invest in.
             */
            try
            {
                if (_Event.parameters.ContainsKey("A"))
                {
                    _CompName = "Microsoft Company";
                    _Investor = _Event.parameters["investor"];

                    /**
                    *@ Checking if any money was invested.
                    */
                    if (string.IsNullOrEmpty(_Event.parameters["name"]) || btnA.IsEnabled == false || string.IsNullOrEmpty(_Event.parameters["investor"]))
                    {
                        user = "0";
                    }
                    else
                    {
                        /**
                        *@ Amount invested.
                        */
                        user = _Event.parameters["name"];
                        valueA = Convert.ToInt64(user);
                        if (valueA >= 1)
                        {
                            btnA_Click(null, null);
                            BankAccount(_Investor, valueA);
                            Transaction(_Investor, user, _CompName);
                        }
                    }
                }
            }
            catch (Exception)
            {
                listOverDraft.Items.Add(_Investor + " investment was invalid.");
            }

            /**
            *@ Code for the company to invest in.
            */
            try
            {
                if (_Event.parameters.ContainsKey("B"))
                {
                    _CompName = "Apple";
                    _Investor = _Event.parameters["investor"];
                    /**
                    *@ Checking if any money was invested.
                    */
                    if (string.IsNullOrEmpty(_Event.parameters["name"]) || btnB.IsEnabled == false || string.IsNullOrEmpty(_Event.parameters["investor"]))
                    {
                        user = "0";
                    }
                    else
                    {
                        /**
                        *@ Amount invested.
                        */
                        user = _Event.parameters["name"];
                        valueB = Convert.ToInt64(user);
                        if (valueB >= 1)
                        {
                            btnB_Click(null, null);
                            BankAccount(_Investor, valueB);
                            Transaction(_Investor, user, _CompName);
                        }
                    }
                }
            }
            catch (Exception)
            {
                listOverDraft.Items.Add(_Investor + " investment was invalid.");
            }
            /**
            *@ Code for the company to invest in.
            */
            try
            {
                if (_Event.parameters.ContainsKey("C"))
                {
                    _CompName = "Nokia";
                    _Investor = _Event.parameters["investor"];
                    /**
                    *@ Checking if any money was invested.
                    */
                    if (string.IsNullOrEmpty(_Event.parameters["name"]) || btnC.IsEnabled == false || string.IsNullOrEmpty(_Event.parameters["investor"]))
                    {
                        user = "0";
                    }
                    else
                    {
                        /**
                        *@ Amount invested.
                        */
                        user = _Event.parameters["name"];
                        valueC = Convert.ToInt64(user);
                        if (valueC >= 1)
                        {
                            btnC_Click(null, null);
                            BankAccount(_Investor, valueC);
                            Transaction(_Investor, user, _CompName);
                        }
                    }
                }
            }
            catch (Exception)
            {
                listOverDraft.Items.Add(_Investor + " investment was invalid.");
            }
            /**
            *@ Code for the company to invest in.
            */
            try
            {
                if (_Event.parameters.ContainsKey("D"))
                {
                    _CompName = "Google";
                    _Investor = _Event.parameters["investor"];
                    /**
                    *@ Checking if any money was invested.
                    */
                    if (string.IsNullOrEmpty(_Event.parameters["name"]) || btnD.IsEnabled == false || string.IsNullOrEmpty(_Event.parameters["investor"]))
                    {
                        user = "0";
                    }
                    else
                    {
                        /**
                         *@ Amount invested.
                         */
                        user = _Event.parameters["name"];
                        valueD = Convert.ToInt64(user);
                        if (valueD >= 1)
                        {
                            btnD_Click(null, null);
                            BankAccount(_Investor, valueD);
                            Transaction(_Investor, user, _CompName);
                        }
                    }
                }
            }
            catch (Exception)
            {
                listOverDraft.Items.Add(_Investor + " investment was invalid.");
            }
            /**
            *@ Code for the company to invest in.
            */
            try
            {
                if (_Event.parameters.ContainsKey("E"))
                {
                    _CompName = "Samsung";
                    _Investor = _Event.parameters["investor"];
                    /**
                    *@ Checking if any money was invested.
                    */
                    if (string.IsNullOrEmpty(_Event.parameters["name"]) || btnE.IsEnabled == false || string.IsNullOrEmpty(_Event.parameters["investor"]))
                    {
                        user = "0";
                    }
                    else
                    {
                        /**
                         *@ Amount invested.
                         */
                        user = _Event.parameters["name"];
                        valueE = Convert.ToInt64(user);
                        if (valueE >= 1)
                        {
                            btnE_Click(null, null);
                            BankAccount(_Investor, valueE);
                            Transaction(_Investor, user, _CompName);
                        }
                    }
                }
            }
            catch (Exception)
            {
                listOverDraft.Items.Add(_Investor + " investment was invalid.");
            }

            listOverDraft.SelectedIndex = listOverDraft.Items.Count - 1;
            listOverDraft.ScrollIntoView(listOverDraft.SelectedItem);
        }

        /**
        *@METHOD			:btnA_Click
        *@DESCRIPTION	:Displays how many times 'A' is chosen for an answer.
        */

        private void btnA_Click(object sender, RoutedEventArgs e)
        {
            if (companyA < 25)
            {
                /**
                 *@ Company ID.
                 */
                _CompID = "A";
                /**
                 *@ increment the number of transaction.
                 */
                companyA++;
                /**
                 *@ Number of transaction.
                 */
                txtA.Text = Convert.ToString(companyA);
                /** *@ Amount invested.
                */
                valueA1 += valueA;
                txtInvestA.Text = "$" + Convert.ToString(valueA1);
                /**
                *@ Calculate profit.
                */
                profit(_CompID, valueA1);
            }
            else { btnA.IsEnabled = false; }
        }

        /**
       *@METHOD			:btnB_Click
       *@DESCRIPTION	:Displays how many times 'B' is chosen for an answer.
       */

        private void btnB_Click(object sender, RoutedEventArgs e)
        {
            if (companyB < 25)
            {
                /**
                 *@ Company ID.
                 */
                _CompID = "B";
                /**
                 *@ increment the number of transaction.
                 */
                companyB++;
                /**
                *@ Number of transaction.
                */
                txtB.Text = Convert.ToString(companyB);
                /**
                *@ Amount invested.
                */
                valueB1 += valueB;
                txtInvestB.Text = "$" + Convert.ToString(valueB1);
                /**
                *@ Calculate profit.
                */
                profit(_CompID, valueB1);
            }
            else
            {
                btnB.IsEnabled = false;
            }
        }

        /**
         *@METHOD			:btnC_Click
         *@DESCRIPTION	:Displays how many times 'C' is chosen for an answer.
         */

        private void btnC_Click(object sender, RoutedEventArgs e)
        {
            if (companyC < 25)
            {
                /**
                 *@ Company ID.
                 */
                _CompID = "C";
                /**
                 *@ increment the number of transaction.
                 */
                companyC++;
                /**
                *@ Number of transaction.
                */
                txtC.Text = Convert.ToString(companyC);
                /**
                *@ Amount invested.
                */
                valueC1 += valueC;
                txtInvestC.Text = "$" + Convert.ToString(valueC1);
                /**
                *@ Calculate profit.
                */
                profit(_CompID, valueC1);
            }
            else
            {
                btnC.IsEnabled = false;
            }
        }

        /**
         *@METHOD			:btnD_Click
         *@DESCRIPTION	:Displays how many times 'D' is chosen for an answer.
         */

        private void btnD_Click(object sender, RoutedEventArgs e)
        {
            if (companyD < 25)
            {
                /**
                 *@ Company ID.
                 */
                _CompID = "D";
                /**
                 *@ increment the number of transaction.
                 */
                companyD++;
                /**
                *@ Number of transaction.
                */
                txtD.Text = Convert.ToString(companyD);
                /**
                *@ Amount invested.
                */
                valueD1 += valueD;
                txtInvestD.Text = "$" + Convert.ToString(valueD1);
                /**
                *@ Calculate profit.
                */
                profit(_CompID, valueD1);
            }
            else
            {
                btnD.IsEnabled = false;
            }
        }

        /**
         *@METHOD			:btnE_Click
         *@DESCRIPTION	:Displays how many times 'E' is chosen for an answer.
         */

        private void btnE_Click(object sender, RoutedEventArgs e)
        {
            if (companyE < 25)
            {
                /**
                 *@ Company ID.
                 */
                _CompID = "E";
                /**
                 *@ increment the number of transaction.
                 */
                companyE++;
                /**
                *@ Number of transaction.
                */
                txtE.Text = Convert.ToString(companyE);
                /**
                *@ Amount invested.
                */
                valueE1 += valueE;
                txtInvestE.Text = "$" + Convert.ToString(valueE1);
                /**
                *@ Calculate profit.
                */
                profit(_CompID, valueE1);
            }
            else
            {
                btnE.IsEnabled = false;
            }
        }

        /**
        *@METHOD			:profit
        *@DESCRIPTION	:Calculates company's profit.
        *@PARAMETERS		:(string _ID, long _Profit)
        *@RETURNS		:None
        */

        public void profit(string _ID, long _Profit)
        {
            /**
             *@ Variables for profit calculation.
             */
            double _val = 0;
            double _val2 = 0;
            long percentage = GetCompanyProfit();
            /**
           *@ Calculate company profit.
           */
            if (_Profit < 70)
            {
                _val = 0;
                _val2 = _val + _Profit;
            }
            if (_Profit > 100)
            {
                _val = _Profit + percentage;
                _val2 = _val + _Profit;
            }

            /**
           *@ Assign Calculated profit to the correct company.
           */
            /**
              *@ Microsoft.
              */
            if (_ID == "A")
            {
                aValue = _val2;
                aProfit = _val;
                txtValueA.Text = "$" + Convert.ToString(_val2);
                txtProfitsA.Text = "$" + Convert.ToString(_val);
            }
            /**
             *@ Apple.
             */
            if (_ID == "B")
            {
                bValue = _val2;
                bProfit = _val;
                txtValueB.Text = "$" + Convert.ToString(_val2);
                txtProfitsB.Text = "$" + Convert.ToString(_val);
            }
            /**
             *@ Nokia.
             */
            if (_ID == "C")
            {
                cValue = _val2;
                cProfit = _val;
                txtValueC.Text = "$" + Convert.ToString(_val2);
                txtProfitsC.Text = "$" + Convert.ToString(_val);
            }
            /**
             *@ Google.
             */
            if (_ID == "D")
            {
                dValue = _val2;
                dProfit = _val;
                txtValueD.Text = "$" + Convert.ToString(_val2);
                txtProfitsD.Text = "$" + Convert.ToString(_val);
            }
            /**
             *@ Samsung.
             */
            if (_ID == "E")
            {
                eValue = _val2;
                eProfit = _val;
                txtValueE.Text = "$" + Convert.ToString(_val2);
                txtProfitsE.Text = "$" + Convert.ToString(_val);
            }
        }

        /**
         *@METHOD			:Transaction
         *@DESCRIPTION	:Displays transaction history.
         *@PARAMETERS		:(string _Investor, string _Amount, string _Company)
         *@RETURNS		:None
         */

        public void Transaction(string _Investor, string _Amount, string _Company)
        {
            var _InvestorA = _Investor;
            var _AmountA = _Amount;
            var _CompanyA = _Company;
            var theAmount = Convert.ToInt64(_AmountA);
            /**
              *@ Checks for a valid transaction.
              */
            if (_InvestorA != "")
            {
                if (_AmountA != "0")
                {
                    /**
                     *@ The name of the investor.
                     */
                    /**
                    *@ The amount that is invested per transaction.
                    */
                    /**
                   *@ The company that was invested in.
                   */
                    listIn.Items.Add(_InvestorA + "\t\t$" + _Amount + "\t\t" + _Company);
                    listIn.SelectedIndex = listIn.Items.Count - 1;
                    listIn.ScrollIntoView(listIn.SelectedItem);

                    /**
                   *@ Redraws the chart.
                   */
                    TheChart();
                    /**
                   *@ Calls the method that calculate investor's profit.
                   */
                    InvertorsProfit(_InvestorA, theAmount, _CompanyA);

                    /**
                    *@ Add investor information to dataset
                    */
                    transactionInfoTable.Rows.Add(_InvestorA, _AmountA, _CompanyA, _TransactionAmount);
                }
            }
        }

        /**
         *@METHOD			:WriteTransactionInfo
         *@DESCRIPTION	:Writes the transaction history to a file
         *@PARAMETERS		:None
         *@RETURNS		:None
         */

        public void WriteTransactionInfo()
        {
            StreamWriter sw = new StreamWriter(@"Transaction Info.txt");

            foreach (DataRow row in transactionInfoTable.Rows)
            {
                object[] array = row.ItemArray;
                int j;
                for (j = 0; j < array.Length - 1; j++)
                {
                    sw.Write(array[j].ToString() + "|");
                }
                sw.Write(array[j].ToString());
                sw.WriteLine();
            }
            sw.Close();
        }

        /**
         *@METHOD			:ReadTransactionInfo
         *@DESCRIPTION	:Reads the transaction history from a file and updates any displayed
         *                 information as necessary
         *@PARAMETERS		:None
         *@RETURNS		:None
         */

        public void ReadTransactionInfo()
        {
            /**
            *@ Check if the file exists
            */
            if (File.Exists(@"Transaction Info.txt"))
            {
                StreamReader reader = File.OpenText(@"Transaction Info.txt");
                string tempInput = "";
                Char delimeter = '|';

                while ((tempInput = reader.ReadLine()) != null)
                {
                    /**
                    *@Read the data from each line of the file into separate elements of a list
                    */
                    List<string> transactionInfo = new List<string>();
                    string id = "";
                    double amount = 0;
                    double bankAmount = 0;
                    string[] substrings = tempInput.Split(delimeter);
                    foreach (var substring in substrings)
                    {
                        transactionInfo.Add(substring);
                    }

                    if (transactionInfo != null)
                    {
                        /**
                        *@ Convert the invested amount into a double
                         */
                        amount = Convert.ToDouble(transactionInfo[1]);
                        bankAmount = Convert.ToDouble(transactionInfo[3]);

                        /**
                        *@ Check which company the money was invested in
                        */
                        if (transactionInfo[2] == "Microsoft Company")
                        {
                            id = "A";
                            valueA1 += (long)amount;
                            companyA++;
                            txtInvestA.Text = "$" + Convert.ToString(valueA1);
                            txtA.Text = Convert.ToString(companyA);
                        }
                        else if (transactionInfo[2] == "Apple")
                        {
                            id = "B";
                            valueB1 += (long)amount;
                            companyB++;
                            txtInvestB.Text = "$" + Convert.ToString(valueB1);
                            txtB.Text = Convert.ToString(companyB);
                        }
                        else if (transactionInfo[2] == "Nokia")
                        {
                            id = "C";
                            valueC1 += (long)amount;
                            companyC++;
                            txtInvestC.Text = "$" + Convert.ToString(valueC1);
                            txtC.Text = Convert.ToString(companyC);
                        }
                        else if (transactionInfo[2] == "Google")
                        {
                            id = "D";
                            valueD1 += (long)amount;
                            companyD++;
                            txtInvestD.Text = "$" + Convert.ToString(valueD1);
                            txtD.Text = Convert.ToString(companyD);
                        }
                        else if (transactionInfo[2] == "Samsung")
                        {
                            id = "E";
                            valueE1 += (long)amount;
                            companyE++;
                            txtInvestE.Text = "$" + Convert.ToString(valueE1);
                            txtE.Text = Convert.ToString(companyE);
                        }

                        /**
                        *@ Call the profit(), transaction(), and BankAccount() methods to update information
                        */
                        _TransactionAmount = Convert.ToString(bankAmount);
                        profit(id, (long)amount);
                        Transaction(transactionInfo[0], transactionInfo[1], transactionInfo[2]);
                        BankAccount(transactionInfo[0], ((long)bankAmount));
                    }
                }
                reader.Close();
            }
        }

        /**
         *@METHOD		:TheChart()
         *@DESCRIPTION	:Column graph for company profits and net values.
         *@PARAMETERS	:None
         *@RETURNS		:None
         */

        public void TheChart()
        {
            /**
             *@ Microsoft investment, profits, and net value.
             */
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Microsoft Investment", valueA1));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Microsoft Profits", aProfit));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Microsoft", aValue));
            /**
             *@ Apple investment, profits, and net value.
             */
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Apple Investment", valueB1));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Apple Profits", bProfit));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Apple", bValue));
            /**
             *@ Nokia investment, profits, and net value.
             */
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Nokia Investment", valueC1));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Nokia Profits", cProfit));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Nokia", cValue));
            /**
             *@ Google investment, profits, and net value.
             */
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Google Investment", valueD1));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Google Profits", dProfit));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Google", dValue));
            /**
             *@ Samsung investment, profits, and net value.
             */
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Samsung Investment", valueE1));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Samsung Profits", eProfit));
            chartControl1.Diagram.Series[0].Points.Add(new SeriesPoint("Samsung", eValue));
        }

        /**
         *@METHOD		:InvertorsProfit
         *@DESCRIPTION	:Calculates investor's profit.
         *@PARAMETERS	:(string theInvestor, long theAmount, string theCompany)
         *@RETURNS		:None
         */

        public void InvertorsProfit(string theInvestor, long theAmount, string theCompany)
        {
            /**
             *@ Variables for managing investor's profit.
             */
            var investor = theInvestor;
            double calaculate = GetInvestorProfit();
            double amount = 0;
            var company = theCompany;
            double buffer = 0;
            string _NewAmount;

            /**
              *@ Microsoft profit for investors.
              */
            listInvestors.Items.Clear();
            if (company == "Microsoft Company")
            {
                amount = theAmount + calaculate;
                /**
               *@ Clears list box for manual override of data.
               */

                /**
                *@ Checking if investor exist in system.
                */
                if (records.ContainsKey(investor))
                {
                    buffer = Convert.ToInt64(records[investor]);
                    amount = amount + buffer;
                    /**
                    *@ Updating current investor's profit.
                    */
                    _NewAmount = Convert.ToString(amount);
                    records.Remove(investor);
                    records.Add(investor, _NewAmount);
                }
                /**
                *@ Adding new investor
                */
                else
                {
                    records.Add(investor, amount);
                }
                /**
                 *@ Displaying investor records in list box.
                 */
                foreach (DictionaryEntry invest in records)
                {
                    listInvestors.Items.Remove(invest.Key);
                    listInvestors.Items.Add(invest.Key + "\t\t $" + invest.Value);
                }
            }

            /**
              *@ Apple profit for investor.
              */
            if (company == "Apple")
            {
                amount = theAmount + calaculate;

                /**
                *@ Checking if investor exist in system.
                */
                if (records.ContainsKey(investor))
                {
                    buffer = Convert.ToInt64(records[investor]);
                    amount = amount + buffer;
                    /**
                    *@ Updating current investor's profit.
                    */
                    records.Remove(investor);
                    records.Add(investor, amount);
                }
                /**
                *@ Adding new investor
                */
                else
                {
                    records.Add(investor, amount);
                }
                /**
                 *@ Displaying investor records in list box.
                 */
                foreach (DictionaryEntry invest in records)
                {
                    listInvestors.Items.Remove(invest.Key);
                    listInvestors.Items.Add(invest.Key + "\t\t $" + invest.Value);
                }
            }

            /**
              *@ Nokia profits for.
              */
            if (company == "Nokia")
            {
                amount = theAmount + calaculate;

                /**
                *@ Checking if investor exist in system.
                */
                if (records.ContainsKey(investor))
                {
                    buffer = Convert.ToInt64(records[investor]);
                    amount = amount + buffer;
                    /**
                    *@ Updating current investor's profit.
                    */
                    _NewAmount = Convert.ToString(amount);
                    records.Remove(investor);
                    records.Add(investor, _NewAmount);
                }
                /**
                *@ Adding new investor
                */
                else
                {
                    records.Add(investor, amount);
                }
                /**
                 *@ Displaying investor records in list box.
                 */
                foreach (DictionaryEntry invest in records)
                {
                    listInvestors.Items.Remove(invest.Key);
                    listInvestors.Items.Add(invest.Key + "\t\t $" + invest.Value);
                }
            }

            /**
              *@ Google profits for investors.
              */
            if (company == "Google")
            {
                amount = theAmount + calaculate;

                /**
                *@ Checking if investor exist in system.
                */
                if (records.ContainsKey(investor))
                {
                    buffer = Convert.ToInt64(records[investor]);
                    amount = amount + buffer;
                    /**
                    *@ Updating current investor's profit.
                    */
                    _NewAmount = Convert.ToString(amount);
                    records.Remove(investor);
                    records.Add(investor, _NewAmount);
                }
                /**
                *@ Adding new investor
                */
                else
                {
                    records.Add(investor, amount);
                }
                /**
                 *@ Displaying investor records in list box.
                 */
                foreach (DictionaryEntry invest in records)
                {
                    listInvestors.Items.Remove(invest.Key);
                    listInvestors.Items.Add(invest.Key + "\t\t $" + invest.Value);
                }
            }
            /**
              *@ Samsung profits for investors.
              */
            if (company == "Samsung")
            {
                amount = theAmount + calaculate;

                /**
                *@ Checking if investor exist in system.
                */
                if (records.ContainsKey(investor))
                {
                    buffer = Convert.ToInt64(records[investor]);
                    amount = amount + buffer;
                    /**
                    *@ Updating current investor's profit.
                    */
                    _NewAmount = Convert.ToString(amount);
                    records.Remove(investor);
                    records.Add(investor, _NewAmount);
                }
                /**
                *@ Adding new investor
                */
                else
                {
                    records.Add(investor, amount);
                }
                /**
                 *@ Displaying investor records in list box.
                 */
                foreach (DictionaryEntry invest in records)
                {
                    listInvestors.Items.Remove(invest.Key);
                    listInvestors.Items.Add(invest.Key + "\t\t $" + invest.Value);
                }
            }
            listInvestors.SelectedIndex = listInvestors.Items.Count - 1;
            listInvestors.ScrollIntoView(listInvestors.SelectedItem);
        }

        /**
         *@METHOD			:GetRandom
         *@DESCRIPTION	:generates random profit values for investors.
         *@PARAMETERS		:None
         *@RETURNS		:Random(DateTime.Now.Millisecond).Next(50, 500);
         */

        private static int GetInvestorProfit()
        {
            Thread.Sleep(1);
            return new Random(DateTime.Now.Millisecond).Next(50, 500);
        }

        /**
         *@METHOD			:GetRandom
         *@DESCRIPTION	:generates random profit values for investors.
         *@PARAMETERS		:None
         *@RETURNS		:Random(DateTime.Now.Millisecond).Next(100, 700);
         */

        private static int GetCompanyProfit()
        {
            Thread.Sleep(1);
            return new Random(DateTime.Now.Millisecond).Next(100, 700);
        }

        /**
         *@METHOD		:resetbutton_Click
         *@DESCRIPTION	:Resets all stock market data.
         *@PARAMETERS	:(object sender, RoutedEventArgs e)
         *@RETURNS		:None
         */

        private void resetbutton_Click(object sender, RoutedEventArgs e)
        {
            /**
             *@ Clearing hash tables.
             */
            records.Clear();
            banking.Clear();
            /**
             *@ Resets display data.
             */
            txtA.Text = "";
            txtB.Text = "";
            txtC.Text = "";
            txtD.Text = "";
            txtE.Text = "";
            txtInvestA.Text = "$0.00";
            txtInvestB.Text = "$0.00";
            txtInvestC.Text = "$0.00";
            txtInvestD.Text = "$0.00";
            txtInvestE.Text = "$0.00";
            txtProfitsA.Text = "$0.00";
            txtProfitsB.Text = "$0.00";
            txtProfitsC.Text = "$0.00";
            txtProfitsD.Text = "$0.00";
            txtProfitsE.Text = "$0.00";
            txtValueA.Text = "$0.00";
            txtValueB.Text = "$0.00";
            txtValueC.Text = "$0.00";
            txtValueD.Text = "$0.00";
            txtValueE.Text = "$0.00";

            /**
             *@ Variable for investment amount.
             */
            valueA = 0;
            valueB = 0;
            valueC = 0;
            valueD = 0;
            valueE = 0;
            /**
            *@ Variable for total investment.
            */
            valueA1 = 0;
            valueB1 = 0;
            valueC1 = 0;
            valueD1 = 0;
            valueE1 = 0;

            /**
            *@ Variable for total company's profits and net value.
            */
            aValue = 0;
            aProfit = 0;
            bValue = 0;
            bProfit = 0;
            cValue = 0;
            cProfit = 0;
            dValue = 0;
            dProfit = 0;
            eValue = 0;
            eProfit = 0;
            /**
            *@ Clearing list boxes.
            */
            listAccount.Items.Clear();
            listIn.Items.Clear();
            listInvestors.Items.Clear();
            listOverDraft.Items.Clear();
            /**
           *@ Clears and redraws chart.
           */
            chartControl1.Diagram.Series[0].Points.Clear();
            TheChart();

            /**
            *@ Remove the data from dataset
            */
            transactionInfoTable.Clear();

            /**
            *@ Remove the data from the file
            */
            File.WriteAllText(@"Transaction Info", string.Empty);
        }

        /**
         *@METHOD		:txtLock
         *@DESCRIPTION	:Disable the text box to be read only.
         *@PARAMETERS	:None
         *@RETURNS		:None
         */

        private void txtLock()
        {
            txtA.IsEnabled = false;
            txtB.IsEnabled = false;
            txtC.IsEnabled = false;
            txtD.IsEnabled = false;
            txtE.IsEnabled = false;
            txtInvestA.IsEnabled = false;
            txtInvestB.IsEnabled = false;
            txtInvestC.IsEnabled = false;
            txtInvestD.IsEnabled = false;
            txtInvestE.IsEnabled = false;
            txtProfitsA.IsEnabled = false;
            txtProfitsB.IsEnabled = false;
            txtProfitsC.IsEnabled = false;
            txtProfitsD.IsEnabled = false;
            txtProfitsE.IsEnabled = false;
            txtValueA.IsEnabled = false;
            txtValueB.IsEnabled = false;
            txtValueC.IsEnabled = false;
            txtValueD.IsEnabled = false;
            txtValueE.IsEnabled = false;
        }

        /**
         *@METHOD		:BankAccount
         *@DESCRIPTION	:Keep track of investor's bank account.
         *@PARAMETERS	:(string theInvestor, long theAmount)
         *@RETURNS		:None
         */

        private void BankAccount(string theInvestor, long theAmount)
        {
            /**
            *@ Variables for managing investor's profit.
            */
            var investor = theInvestor;
            double amount = 10000;
            double buffer = 0;
            string _NewAmount;

            /**
           *@ Clears list box for manual override of data.
           */
            listAccount.Items.Clear();
            /**
            *@ Checking if investor exist in system.
            */
            if (banking.ContainsKey(investor) && banking[investor] != "[OVER-DRAFT]")
            {
                buffer = Convert.ToInt64(banking[investor]);
                amount = buffer - theAmount;

                if (buffer > 0 && amount > 0)
                {
                    /**
                    *@ Updating current investor's profit.
                    */
                    _NewAmount = Convert.ToString(amount);
                    banking.Remove(investor);
                    banking.Add(investor, _NewAmount);
                }
                else if (buffer < 1 || amount < 1)
                {
                    banking.Remove(investor);
                    banking.Add(investor, "[OVER-DRAFT]");
                    listOverDraft.Items.Add(investor + " has a $" + amount + " over draft please contact their banking institute.");
                }
            }
            /**
            *@ Adding new investor
            */
            else if (banking[investor] != "[OVER-DRAFT]")
            {
                amount = amount - theAmount;
                banking.Add(investor, amount);
            }

            /**
             *@ Displaying investor records in list box.
             */

            foreach (DictionaryEntry invest in banking)
            {
                listAccount.Items.Remove(invest.Key);
                listAccount.Items.Add(invest.Key + "\t\t\t$" + invest.Value);
            }

            listAccount.SelectedIndex = listAccount.Items.Count - 1;
            listAccount.ScrollIntoView(listAccount.SelectedItem);

            listOverDraft.SelectedIndex = listOverDraft.Items.Count - 1;
            listOverDraft.ScrollIntoView(listOverDraft.SelectedItem);

            /**
            *@ Add to variable for use in dataset
            */
            _TransactionAmount = Convert.ToString(theAmount);
        }
    }
}