﻿/**
*@FILE  		: Server.xaml.cs
*@DESCRIPTION	: This class manages the wireless connection
*@              : between the client page and the server.
*@              : DECEMBER 16, 2015
*@VERSION		: 1.0.0
*@PROGRAMMER	: BRIAN HINDS & JAMES SIMPSON
*/
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Media;
using System.Net;
using System.ComponentModel;
using System.IO;

namespace PROG2120_Project
{
    /**
    *@CLASS			    :Server : UserControl
    *@DESCRIPTION		:Interaction logic for Server.xaml
    */
    public partial class Server : UserControl
    {
        /**
        *@ Variable for managing server connection.
        */
        public string _Information = "";
        /*This allows executions on separate threads.*/
        private readonly BackgroundWorker parallelThread;
        /*This provides control for the HTTP listener.*/
        private HttpListener listenForConnection;
        /*This determines the connection status of the server*/
        private bool processCancel = false;
        /*This stores the port number of the server.*/
        private int port = 8000;
        
        /**
        *@ This syntax is used to represent a collection of keys and values.
        */
        static Dictionary<string, string> parameters = new Dictionary<string, string>();
        /**
        *@METHOD			:Server()
        *@DESCRIPTION		:Initialized the components for the server
        */
        public Server()
        {
            InitializeComponent();
        /**
        *@ This syntax searches for a resource with name ResourceName and 
        *@ of type ResourceType in the executable or library identified by ModuleHandle.
        */
            parallelThread = this.FindResource("backgroundWorker") as BackgroundWorker; 
            
        }

        /**
        *@METHOD			:StartServerConnection()
        *@DESCRIPTION		:This method starts the server connection
        *@                  :and setup protocols for client connection.
        *@PARAMETERS		:None
        *@RETURNS			:None
        */
        public void StartServerConnection()
        {
            parallelThread.RunWorkerAsync();

            /**
            *@ These syntax checks the port number preset in the server..
             */
            RemoteServer.Inlines.Clear();
            var portName = "";
            if (port != 8000)
                portName = ":" + port.ToString();
            /**
             *@ These syntax displays the IP or machine name for the clients to connect to..
             */
            var serverIP = Dns.GetHostAddresses(Dns.GetHostName());
            string _IP = null;
            
            foreach (IPAddress ip in serverIP)
                if (ip.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                    _IP = ip.ToString();
            RemoteServer.Inlines.Add("http://104.234.44.183" + portName + " OR http://" + Dns.GetHostName() + portName);
            _Information = ("http://104.234.44.183"  + portName  + "   OR http://" + Dns.GetHostName() + portName);
            /**
            *@ These syntax displays the port number preset in the server..
            */
            if (port == 8000) { _Information = ("http://104.234.44.183"  +":" + port.ToString() + "   OR http://" + Dns.GetHostName() +":" + port.ToString()); }
            
            RemoteServer.Background = new SolidColorBrush(Colors.DarkGray);
        }   
        /**
        *@METHOD			:DisconnectServer()
        *@DESCRIPTION		:This method allows graceful shutdown of the server
        *@                  :without crashing or exception errors.
        *@PARAMETERS		:None
        *@RETURNS			:None
        */
        public void DisconnectServer()
        {
            parallelThread.CancelAsync();
            processCancel = true;
            listenForConnection.Close();
        }
        /**
        *@METHOD			:ParallelThread_Parallel
        *@DESCRIPTION		:This method setup up a listening protocol for clients
        *@                  :then handles the client's page and request revived.
        *@PARAMETERS		:(object sender, System.ComponentModel.DoWorkEventArgs e)
        *@RETURNS			:None
        */
        private void ParallelThread_Parallel(object sender, System.ComponentModel.DoWorkEventArgs e)
        {
            /**
            *@ This syntax creates a listening protocol.
            */
            listenForConnection = new HttpListener();
            /**
            *@ This syntax add the prefixes (port number) for clients' that connects .
            */
            listenForConnection.Prefixes.Add("http://*:" + port.ToString() + "/");

            try
            {
                /**
                *@ This syntax starts the listening protocol.
                 */
                listenForConnection.Start();
                /**
                 *@ These syntax stream in the clients' side HTML page that will be read.
                 */
                using (TextReader StreamText = new StreamReader("client.htm"))
                {
                    var clientPage = StreamText.ReadToEnd();
                    while (!parallelThread.CancellationPending)
                    {
                        /**
                        *@ These syntax allows the GetContext to be block while waiting on a client's request.
                         */
                        var httpContext = listenForConnection.GetContext();
                        if (httpContext.Request.HttpMethod == "POST")
                        {
                            /**
                            *@ These syntax handle the POST method and input parameters from the client.
                             */
                            var clientBody = new StreamReader(httpContext.Request.InputStream).ReadToEnd();
                            ParseClientPostParameters(clientBody);
                            parallelThread.ReportProgress(1);
                        }
                        /**
                         *@ These syntax set up a response object that handle the page.
                         */
                        var objectBuffer = System.Text.Encoding.UTF8.GetBytes(clientPage);
                        httpContext.Response.ContentLength64 = objectBuffer.Length;
                        httpContext.Response.OutputStream.Write(objectBuffer, 0, objectBuffer.Length);
                        httpContext.Response.Close();
                    }

                    listenForConnection.Stop();
                }
            }
            /**
             *@ These syntax throw an exception if the listening protocol failed.
             */
            catch (Exception exct)
            {
                if (!processCancel)
                    MessageBox.Show("Remote Listener Failed. " + exct.Message);
            }
        }
        /**
       *@METHOD			:ParseClientPostParameters
       *@DESCRIPTION	:This method parses the client's POST data
       *@               :sent from the client's page.
       *@PARAMETERS		:(string clientBody)
       *@RETURNS		:None
       */
        private static void ParseClientPostParameters(string clientBody)
        {
            var parseParameter = clientBody.Split('&');
            parameters.Clear();
            foreach (string _String in parseParameter)
            {
                var index = _String.IndexOf('=');
                if (index > - 1)
                {
                    var PrimaryKey = _String.Substring(0, index);
                    var KeyValue = _String.Substring(index + 1);
                    KeyValue = System.Uri.UnescapeDataString(KeyValue);
                    parameters.Add(PrimaryKey, KeyValue);
                }
            }
        }
        /**
       *@METHOD			:ParallelThread_ParallelComplete
       *@DESCRIPTION	:This method handles the situation
       *@               :where the server has stopped.
       *@PARAMETERS		:(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
       *@RETURNS		:None
       */
        private void ParallelThread_ParallelComplete(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            processCancel = false;
            RemoteServer.Inlines.Clear();
            RemoteServer.Inlines.Add("Remote Server Stopped");
            RemoteServer.Background = new SolidColorBrush(Colors.Red);
        }
        /**
      *@METHOD		:ParallelThread_ParallelChanged
      *@DESCRIPTION	:This method provides data needed
      *@            :for the thread progress change event.
      *@PARAMETERS	:(object sender, System.ComponentModel.ProgressChangedEventArgs e)
      *@RETURNS		:None
      */
        private void ParallelThread_ParallelChanged(object sender, System.ComponentModel.ProgressChangedEventArgs e)
        {
            RoutedEventArgs newEventArguments = new RemoteControlEventArgs(parameters);
            RaiseEvent(newEventArguments);
            System.Media.SystemSounds.Asterisk.Play();
        }


        /**
         *@ These syntax generate events.
         */
        public static readonly RoutedEvent ServerControlEvent = EventManager.RegisterRoutedEvent(nameof(PROG2120_Project), RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(UserControl));
        /**
        *@CLASS			    :RemoteControlEventArgs : RoutedEventArgs
        *@DESCRIPTION		:Interaction logic for Remote control event arguments.
         */
        public class RemoteControlEventArgs : RoutedEventArgs
        {
            /**
             *@METHOD		:ParallelThread_ParallelChanged
             *@DESCRIPTION	:This method provides data needed
             *@            :for the thread progress change event.
             *@PARAMETERS	:(Dictionary<string, string> parameters)
             *@RETURNS		:None
             */
            public RemoteControlEventArgs(Dictionary<string, string> parameters)
            {
                RoutedEvent = ServerControlEvent;
                this.parameters = parameters;
            }
            /**
            *@ This syntax is used to represent a collection of keys and values.
             */
            public Dictionary<string, string> parameters;
        }
        /**
        *@METHOD		:RoutedEventHandler PROG2120_Project
        *@DESCRIPTION	:This method provides data needed
        *@              :for the thread progress change event.
        *@PARAMETERS	:None
        *@RETURNS		:None
        */
        public event RoutedEventHandler PROG2120_Project
        {
            add
            {
                AddHandler(ServerControlEvent, value);
            }
            remove
            {
                RemoveHandler(ServerControlEvent, value);
            }
        }
    }
}
